<?php

namespace TNW\Salesforce\Synchronize\Queue;

/**
 * Entity Synchronize
 */
class Synchronize
{
    /**
     * @var int
     */
    private $type;

    /**
     * @var boolean
     */
    private $isCheck = false;

    /**
     * @var \TNW\Salesforce\Synchronize\Queue
     */
    private $synchronizeQueue;

    /**
     * @var \TNW\Salesforce\Model\ResourceModel\Queue\CollectionFactory
     */
    private $collectionQueueFactory;

    /**
     * @var \Magento\Store\Api\WebsiteRepositoryInterface
     */
    private $websiteRepository;

    /**
     * @var \TNW\Salesforce\Model\Config\WebsiteEmulator
     */
    private $websiteEmulator;

    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    private $messageManager;

    /**
     * @var \TNW\Salesforce\Model\Config
     */
    private $salesforceConfig;

    /** @var \Magento\Framework\App\State  */
    private $state;

    /**
     * Queue constructor.
     * @param int $type
     * @param \TNW\Salesforce\Synchronize\Queue $synchronizeQueue
     * @param \TNW\Salesforce\Model\ResourceModel\Queue\CollectionFactory $collectionQueueFactory
     * @param \Magento\Store\Api\WebsiteRepositoryInterface $websiteRepository
     * @param \TNW\Salesforce\Model\Config\WebsiteEmulator $websiteEmulator
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     */
    public function __construct(
        $type,
        \TNW\Salesforce\Synchronize\Queue $synchronizeQueue,
        \TNW\Salesforce\Model\ResourceModel\Queue\CollectionFactory $collectionQueueFactory,
        \Magento\Store\Api\WebsiteRepositoryInterface $websiteRepository,
        \TNW\Salesforce\Model\Config\WebsiteEmulator $websiteEmulator,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \TNW\Salesforce\Model\Config $salesforceConfig,
        \Magento\Framework\App\State $state,
        $isCheck = false
    )
    {
        $this->type = $type;
        $this->synchronizeQueue = $synchronizeQueue;
        $this->collectionQueueFactory = $collectionQueueFactory;
        $this->websiteRepository = $websiteRepository;
        $this->websiteEmulator = $websiteEmulator;
        $this->messageManager = $messageManager;
        $this->salesforceConfig = $salesforceConfig;
        $this->state = $state;
        $this->setIsCheck($isCheck);
    }

    /**
     * @return bool
     */
    public function isCheck()
    {
        return $this->isCheck;
    }

    /**
     * @param bool $isCheck
     */
    public function setIsCheck(bool $isCheck)
    {
        $this->isCheck = $isCheck;
    }

    /**
     * Sync Type
     *
     * @return int
     */
    public function type()
    {
        return $this->type;
    }

    /**
     * Synchronize
     *
     * @throws \Exception
     */
    public function synchronize()
    {
        foreach ($this->websiteRepository->getList() as $website) {
            $this->websiteEmulator->wrapEmulationWebsite([$this, 'synchronizeToWebsite'], $website->getId());
        }
    }

    /**
     * Synchronize To Website
     *
     * @param int $websiteId
     * @throws \Exception
     */
    public function synchronizeToWebsite($websiteId)
    {
        $collection = $this->collectionQueueFactory->create()
            ->addFilterToSyncType($this->type);

        $collection->addFieldToFilter('main_table.sync_attempt', ['lt' => $this->salesforceConfig->getSyncMaxAttemptsCount()]);

        try {
            $this->synchronizeQueue->synchronize($collection, $websiteId);
        } catch (\Exception $e) {
            if ($this->state->getAreaCode() == \Magento\Framework\App\Area::AREA_ADMINHTML) {
                $this->messageManager->addErrorMessage($e->getMessage());
            }
        } finally {
            if ($collection->count() > 0 && !in_array(false, $collection->walk('isSuccess'), true)) {
                if ($this->state->getAreaCode() == \Magento\Framework\App\Area::AREA_ADMINHTML) {
                    $this->messageManager->addSuccessMessage('All records were successfully synchronized with Salesforce.');
                }
            }

            if ($this->type === \TNW\Salesforce\Model\Config::DIRECT_SYNC_TYPE_REALTIME) {
                foreach ($collection as $queue) {
                    $collection->getResource()->delete($queue);
                }
            }
        }
    }
}
